function output_matrix = raw_to_filled_point_cloud(file,resolution,radius)

% Hole fill raw CT scan slices as part of alpha shapes methods
% Example code for a single baculum
% Gardiner, Benson and Brassey 2018
% j.d.gardiner@liverpool.ac.uk


% Use filename to work out the size of the file so that it can be reshaped
% into a stack of slices

size_index = find(file == '-',2);
size_string = file((size_index(1)+1):(size_index(2)-1));
size_numbers = str2num(regexprep(size_string,'x',' '));

% work out species name for naming hole check figures

species_index_start = find(file == '_',1);
species_index_end = find(file == '-',1);
species = file((species_index_start+1):(species_index_end-1));



% open file and read raw data
fid = fopen(fullfile(file));
data_raw = fread(fid,prod(size_numbers),'uint8');
fclose(fid);


% reshape data into stack

data_stacked = reshape(data_raw, size_numbers);


% remove empty slices from stack

count = 0;

for i = 1:size(data_stacked,3)
    
    one_slice = data_stacked(:,:,i);
    index = one_slice>0;
    
    if sum(index) == 0
        
    else
        
        count = count+1;
        
        data_binary(:,:,count) = index;
        
        
    end
    
end



% fill holes in individual slices

se = strel('disk',radius); % set the size of the disk for hole filling

figure('color','w')


for i = 1:size(data_binary,3)
    
    one_slice = data_binary(:,:,i);
    index = one_slice>0;
    data_gaps_closed_only(:,:,i) = imclose(index,se);
    data_gaps_closed_holes_filled(:,:,i) = imfill(data_gaps_closed_only(:,:,i),'holes');
    data_holes_filled_only(:,:,i) = imfill(index,'holes');
    
    if i == 170
        
        colormap('bone')
        
        subplot(2,2,1)
        
        imagesc(one_slice)
        axis equal
        axis off
        
        subplot(2,2,2)
        
        imagesc(data_gaps_closed_holes_filled(:,:,i))
        axis equal
        axis off
        
    end
    
end



% convert slices into point cloud


for i = 1:length(data_gaps_closed_holes_filled(1,1,:))
    
    one_slice = data_gaps_closed_holes_filled(:,:,i); % call in slice
    
    [X,Y] = find(one_slice);
    
    X_scaled = X.*resolution;
    Y_scaled = Y.*resolution;
    Z_scaled = i.*ones(length(X_scaled),1).*resolution;
    
    if i == 1
        
        X_scaled_all = X_scaled;
        Y_scaled_all = Y_scaled;
        Z_scaled_all = Z_scaled;
        
    else
        
        X_scaled_all = [X_scaled_all;X_scaled];
        Y_scaled_all = [Y_scaled_all;Y_scaled];
        Z_scaled_all = [Z_scaled_all;Z_scaled];
        
    end
end


% output point cloud

output_matrix = [X_scaled_all,Y_scaled_all,Z_scaled_all];



% produce check to see if hole filling has worked properly

pixels_per_slice_binary = squeeze(sum(sum(data_binary)));
pixels_per_slice_gaps_closed_only = squeeze(sum(sum(data_gaps_closed_only)));
pixels_per_slice_holes_filled_only = squeeze(sum(sum(data_holes_filled_only)));
pixels_per_slice_gaps_closed_holes_filled = squeeze(sum(sum(data_gaps_closed_holes_filled)));


subplot(2,1,2)

h(1) = plot(pixels_per_slice_gaps_closed_holes_filled);
hold on
h(2) = plot(pixels_per_slice_gaps_closed_only);
h(3) = plot(pixels_per_slice_holes_filled_only);
h(4) = plot(pixels_per_slice_binary);

for i = 1:4
    set(h(i),'linewidth',1)
end

xlabel('slice')
ylabel('pixels')
box off
h = legend ('gaps close and holes filled','gaps closed only','holes filled only','original');
set(h,'box','off')
aaa = axis;
aaa(4) = 6000;
axis(aaa);
set(gca,'fontsize',12)



end